//  K8055-MARIE JAVASCRIPT - ver. 3.1.1 - for App K8055-MARIE ver. 2.72a
// 
// JavaScript library for remote control Velleman USB board K8055(N) 
// This script needs the jQuery library for your run, please insert its implementation before calling the library K8055.js.
// The functions written to the K8055 are solved asynchronously, so they do not return any value after calling, and they release the handler immediately.
// The function that returns the value of the K8055 board must wait for a response of the board (temporarily stops running the next part of the program).
// The special feature is the K8055_Json function, which reads the overall state of the K8055 board into super-variable card[].
// This load is asynchronous - it can be used for time-periodically loading the status of the K8055 in the background (filling the variable card []). 
// The status of the K8055 does not need to be read by slow synchronous functions, but direct queries on individual super-variable items.
// It allows to use asynchronous techniques for reading values from the board.
// BUT: Direct use of function K8055_Json requires an more sophisticated approach to creating a program!
// =================================================

if (!MARIE_URL) {var MARIE_URL = 'http://127.0.0.1:8080/';}; // MARIE_URL must be defined before this script
var urlCOM = MARIE_URL + '/MARIE-COMMAND/CARD'; 
var strReturn;

var card = new Array();
card[0] = {"OUT":{"ANALOG":{"A":0,"B":0},"DIGITAL":0},"IN":{"ANALOG":{"A":0,"B":0},"DIGITAL":0},"COUNTER":{"A":0,"B":0}};
card[1] = {"OUT":{"ANALOG":{"A":0,"B":0},"DIGITAL":0},"IN":{"ANALOG":{"A":0,"B":0},"DIGITAL":0},"COUNTER":{"A":0,"B":0}};
card[2] = {"OUT":{"ANALOG":{"A":0,"B":0},"DIGITAL":0},"IN":{"ANALOG":{"A":0,"B":0},"DIGITAL":0},"COUNTER":{"A":0,"B":0}};
card[3] = {"OUT":{"ANALOG":{"A":0,"B":0},"DIGITAL":0},"IN":{"ANALOG":{"A":0,"B":0},"DIGITAL":0},"COUNTER":{"A":0,"B":0}};

function TestCardAddr() {
	var stav = true;
	if (typeof CurrentDevice === "undefined") {
		alert("CAUTION:\nThe variable \"CurrentDevice\" was not be set before calling the remote control commands.\nFirst of all, use the procedure \"CurrentDevice()\".\n\nRemote access to board K8055 IS NOT ABLE TO OPERATE!!!");
		stav = false;
	};
	if ((CurrentDevice !== 0)&&(CurrentDevice !== 1)&&(CurrentDevice !== 2)&&(CurrentDevice !== 3)) {
		alert("CAUTION:\nInvalid value of the K8055 address in the variable \"CurrentDevice\".\n\nRemote access to board K8055 IS NOT ABLE TO OPERATE!!!");
		stav = false;
	};
return stav;
}

function SetCurrentDevice(Device) {
if ((Device !== 0)&&(Device !== 1)&&(Device !== 2)&&(Device !== 3)) {
		alert("CAUTION:\nInvalid value of the K8055 address in the variable \"CurrentDevice\".\n\nRemote access to board K8055 IS NOT ABLE TO OPERATE!!!");
	} 
	else {
		CurrentDevice = Device;	
	};
}

// ------ NACITANI ------
function K8055_Json(cardAdr, callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM + cardAdr + '/jsonstatus()',
		async: true,
		success: function(zprava) { 
			card[cardAdr] = jQuery.parseJSON(zprava);
			if (callback) {callback(zprava);}; 
		}
	})
  }
}

function SearchDevices(callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/searchdevices()',
		async: false,
		success: function(zprava) {
			strReturn = zprava;
			if (callback) {callback(zprava);}
		}
        })
	return strReturn;
  }
}

function ReadAnalogChannel(Channel, callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM + CurrentDevice + '/readanalogchannel('+Channel+')',
		async: false,
		success: function(zprava) {
			if (zprava != 'Error') {
				if (Channel == 1) {card[CurrentDevice].IN.ANALOG.A = zprava;} else {card[CurrentDevice].IN.ANALOG.B = zprava;};
			}
			strReturn = zprava;
			if (callback) {callback(zprava);}
		}
        })
	return strReturn;
  }
}

function ReadAllAnalog(callback) {
var hod = new Array(2);
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/readallanalog()',
		async: false,
		success: function(zprava) {
			if (zprava != 'Error') {
				hod = zprava.split('|');
				if (hod.length > 1) {card[CurrentDevice].IN.ANALOG.A = hod[0]; card[CurrentDevice].IN.ANALOG.B = hod[1];}
			}
			strReturn = zprava;
			if (callback) {callback(zprava);}
		}
	})
	return strReturn;
  }
}

function ReadDigitalChannel(Channel, callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/readdigitalchannel('+Channel+')',
		async: false,
		success: function(zprava) {
			if (zprava != 'Error') {
				switch (Channel) {
					case 1 : if (zprava == '1') {card[CurrentDevice].IN.DIGITAL = card[CurrentDevice].IN.DIGITAL | 1;};
						 if (zprava == '0') {card[CurrentDevice].IN.DIGITAL = card[CurrentDevice].IN.DIGITAL & 30;};
						 break;
					case 2 : if (zprava == '1') {card[CurrentDevice].IN.DIGITAL = card[CurrentDevice].IN.DIGITAL | 2;};
						 if (zprava == '0') {card[CurrentDevice].IN.DIGITAL = card[CurrentDevice].IN.DIGITAL & 29;};
						 break;
					case 3 : if (zprava == '1') {card[CurrentDevice].IN.DIGITAL = card[CurrentDevice].IN.DIGITAL | 4;};
					  	 if (zprava == '0') {card[CurrentDevice].IN.DIGITAL = card[CurrentDevice].IN.DIGITAL & 27;};
						 break;
					case 4 : if (zprava == '1') {card[CurrentDevice].IN.DIGITAL = card[CurrentDevice].IN.DIGITAL | 8;};
						 if (zprava == '0') {card[CurrentDevice].IN.DIGITAL = card[CurrentDevice].IN.DIGITAL & 23;};
						 break;
					case 5 : if (zprava == '1') {card[CurrentDevice].IN.DIGITAL = card[CurrentDevice].IN.DIGITAL | 16;};
						 if (zprava == '0') {card[CurrentDevice].IN.DIGITAL = card[CurrentDevice].IN.DIGITAL & 15;};
						 break;
				}
			}
			strReturn = zprava;
			if (callback) {callback(zprava);}
		}
	})
	return strReturn;
  }
}

function ReadAllDigital(callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/readalldigital()',
		async: false,
		success: function(zprava) {
			if (zprava != 'Error') {
				card[CurrentDevice].IN.DIGITAL = zprava;
			}
			strReturn = zprava;
			if (callback) {callback(zprava);}
		}
	})
	return strReturn;
  }
}

function ReadCounter(CounterNr, callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/readcounter('+CounterNr+')',
		async: false,
		success: function(zprava) {
			if (zprava != 'Error') {
				if (CounterNr == 1) {card[CurrentDevice].COUNTER.A = zprava;} else {card[CurrentDevice].COUNTER.B = zprava;};
			}
			strReturn = zprava;
			if (callback) {callback(zprava);}
		}
	})
	return strReturn;
  }
}

function ReadBackDigitalOUT(callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/readbackdigitalout()',
		async: false,
		success: function(zprava) {
			if (zprava != 'Error') {
				card[CurrentDevice].OUT.DIGITAL = zprava;
			}
			strReturn = zprava;
			if (callback) {callback(zprava);}
		}
	})
	return strReturn;
  }
}

function ReadBackAnalogOUT(callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/readbackanalogout()',
		async: false,
		success: function(zprava) {
			if (zprava != 'Error') {
				hod = zprava.split('|');
				if (hod.length > 1) {card[CurrentDevice].OUT.ANALOG.A = hod[0]; card[CurrentDevice].OUT.ANALOG.B = hod[1];}
			}
			strReturn = zprava;
			if (callback) {callback(zprava);}
		}
	})
	return strReturn;
  }
}

// ------ ZAPIS ------
function OutputAnalogChannel(Channel, Data, callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/outputanalogchannel('+Channel+','+Data+')',
		async: true,
		success: function(zprava) { 
			if (Channel == 1) {card[CurrentDevice].OUT.ANALOG.A = Data;} else {card[CurrentDevice].OUT.ANALOG.B = Data;};
			if (callback) {callback(zprava);}
		}
	})
  }
}

function SetAnalogChannel(Channel, callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/setanalogchannel('+Channel+')',
		async: true,
		success: function(zprava) { 
			if (Channel == 1) {card[CurrentDevice].OUT.ANALOG.A = 255;} else {card[CurrentDevice].OUT.ANALOG.B = 255;};
			if (callback) {callback(zprava);}
		}
	})
  }
}

function ClearAnalogChannel(Channel, callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/clearanalogchannel('+Channel+')',
		async: true,
		success: function(zprava) { 
			if (Channel == 1) {card[CurrentDevice].OUT.ANALOG.A = 0;} else {card[CurrentDevice].OUT.ANALOG.B = 0;};
			if (callback) {callback(zprava);}
		}
	})
  }
}

function SetAllAnalog(callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/setallanalog()',
		async: true,
		success: function(zprava) { 
				card[CurrentDevice].OUT.ANALOG.A = 255;
				card[CurrentDevice].OUT.ANALOG.B = 255;
				if (callback) {callback(zprava);}
		}
	})
  }
}

function ClearAllAnalog(callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/clearallanalog()',
		async: true,
		success: function(zprava) { 
			card[CurrentDevice].OUT.ANALOG.A = 0;
			card[CurrentDevice].OUT.ANALOG.B = 0;
			if (callback) {callback(zprava);}
		}
	})
  }
}

function OutputAllAnalog(Data1, Data2, callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/outputallanalog('+Data1+','+Data2+')',
		async: true,
		success: function(zprava) { 
			card[CurrentDevice].OUT.ANALOG.A = Data1;
			card[CurrentDevice].OUT.ANALOG.B = Data2;
			if (callback) {callback(zprava);}
		}
	})
  }
}

function SetDigitalChannel(Channel, callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/setdigitalchannel('+Channel+')',
		async: true,
		success: function(zprava) {
			switch (Channel) {
				case 1 : card[CurrentDevice].OUT.DIGITAL = card[CurrentDevice].OUT.DIGITAL | 1; break;
				case 2 : card[CurrentDevice].OUT.DIGITAL = card[CurrentDevice].OUT.DIGITAL | 2; break;
				case 3 : card[CurrentDevice].OUT.DIGITAL = card[CurrentDevice].OUT.DIGITAL | 4; break;
				case 4 : card[CurrentDevice].OUT.DIGITAL = card[CurrentDevice].OUT.DIGITAL | 8; break;
				case 5 : card[CurrentDevice].OUT.DIGITAL = card[CurrentDevice].OUT.DIGITAL | 16; break;
				case 6 : card[CurrentDevice].OUT.DIGITAL = card[CurrentDevice].OUT.DIGITAL | 32; break;
				case 7 : card[CurrentDevice].OUT.DIGITAL = card[CurrentDevice].OUT.DIGITAL | 64; break;
				case 8 : card[CurrentDevice].OUT.DIGITAL = card[CurrentDevice].OUT.DIGITAL | 128; break;
			}
			if (callback) {callback(zprava);}
		}
	})
  }
}

function ClearDigitalChannel(Channel, callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/cleardigitalchannel('+Channel+')',
		async: true,
		success: function(zprava) { 
			switch (Channel) {
		 		case 1 : card[CurrentDevice].OUT.DIGITAL = card[CurrentDevice].OUT.DIGITAL & 254; break;
				case 2 : card[CurrentDevice].OUT.DIGITAL = card[CurrentDevice].OUT.DIGITAL & 253; break;
				case 3 : card[CurrentDevice].OUT.DIGITAL = card[CurrentDevice].OUT.DIGITAL & 251; break;
				case 4 : card[CurrentDevice].OUT.DIGITAL = card[CurrentDevice].OUT.DIGITAL & 247; break;
				case 5 : card[CurrentDevice].OUT.DIGITAL = card[CurrentDevice].OUT.DIGITAL & 239; break;
				case 6 : card[CurrentDevice].OUT.DIGITAL = card[CurrentDevice].OUT.DIGITAL & 223; break;
			 	case 7 : card[CurrentDevice].OUT.DIGITAL = card[CurrentDevice].OUT.DIGITAL & 191; break;
			 	case 8 : card[CurrentDevice].OUT.DIGITAL = card[CurrentDevice].OUT.DIGITAL & 127; break;
			}
			if (callback) {callback(zprava);}
		}
	})
  }
}

function SetAllDigital(callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/setalldigital()',
		async: true,
		success: function(zprava) { 
			card[CurrentDevice].OUT.DIGITAL = 255;
			if (callback) {callback(zprava);}
		}
	})
  }
}

function ClearAllDigital(callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/clearalldigital()',
		async: true,
		success: function(zprava) { 
			card[CurrentDevice].OUT.DIGITAL = 0;
			if (callback) {callback(zprava);}
		}
	})
  }
}

function WriteAllDigital(Data, callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/writealldigital('+Data+')',
		async: true,
		success: function(zprava) { 
			card[CurrentDevice].OUT.DIGITAL = Data;
			if (callback) {callback(zprava);}
		}
	})
  }
}

function ResetCounter(CounterNr, callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/resetcounter('+CounterNr+')',
		async: true,
		success: function(zprava) { 
			if (CounterNr == 1) {card[CurrentDevice].COUNTER.A = 0;} else {card[CurrentDevice].COUNTER.B = 0;};
			if (callback) {callback(zprava);}
		}
	})
  }
}

function SetCounterDebounceTime(CounterNr, TimeDebounceTime, callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/setcounterdebouncetime('+CounterNr+','+TimeDebounceTime+')',
		async: true,
		success: function(zprava) { 
			if (callback) {callback(zprava);}
		}
	})
  }
}

function GeneratorType(AnalOut,Typ, callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/generatortype('+AnalOut+','+Typ+')',
		async: true,
		success: function(zprava) { 
			if (callback) {callback(zprava);}
		}
	})
  }
}

function GeneratorFrequency(AnalOut, Freq, callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/generatorfrequency('+AnalOut+','+Freq+')',
		async: true,
		success: function(zprava) { 
			if (callback) {callback(zprava);}
		}
	})
  }
}

function GeneratorTimeInterval(AnalOut, Time, callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/generatortimeinterval('+AnalOut+','+Time+')',
		async: true,
		success: function(zprava) { 
			if (callback) {callback(zprava);}
		}
	})
  }
}

function GeneratorStart(AnalOut, callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/generatorstart('+AnalOut+')',
		async: true,
		success: function(zprava) { 
			if (callback) {callback(zprava);}
		}
	})
  }
}

function GeneratorStop(AnalOut, callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/generatorstop('+AnalOut+')',
		async: true,
		success: function(zprava) { 
			if (callback) {callback(zprava);}
		}
	})
  }
}

// ------ zpetna kompatibilita ------
function ReadBothCounters(callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/readbothcounters()',
		async: false,
		success: function(zprava) {
			if (zprava != 'Error') {
				hod = zprava.split('|');
				if (hod.length > 1) {card[CurrentDevice].COUNTER.A = hod[0]; card[CurrentDevice].COUNTER.B = hod[1];}
			}
			strReturn = zprava;
			if (callback) {callback(zprava);}
		}
	})
	return strReturn;
  }
}

function ReadAllInputs(callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/readallinputs()',
		async: false,
		success: function(zprava) {
			if (zprava != 'Error') {
				hod = zprava.split('|');
				if (hod.length > 1) {
					card[CurrentDevice].IN.ANALOG.A = hod[0]; card[CurrentDevice].IN.ANALOG.B = hod[1];
					card[CurrentDevice].IN.DIGITAL = hod[2];
					card[CurrentDevice].IN.ANALOG.A = hod[3]; card[CurrentDevice].IN.ANALOG.B = hod[4];
				}
			}
			strReturn = zprava;
			if (callback) {callback(zprava);}
		}
	})
	return strReturn;
  }
}

function ReadStatus(callback) {
  if (TestCardAddr()) {
	$.ajax({
		type: 'GET',
		cache: false,
		crossDomain: true,
		url: urlCOM +  CurrentDevice + '/readstatus()',
		async: false,
		success: function(zprava) {
			if (zprava != 'Error') {
				hod = zprava.split('|');
				if (hod.length > 1) {
					card[CurrentDevice].IN.ANALOG.A = hod[0]; card[CurrentDevice].IN.ANALOG.B = hod[1];
					card[CurrentDevice].IN.DIGITAL = hod[2];
					card[CurrentDevice].IN.ANALOG.A = hod[3]; card[CurrentDevice].IN.ANALOG.B = hod[4];
					card[CurrentDevice].IN.ANALOG.A = hod[5]; card[CurrentDevice].IN.ANALOG.B = hod[6];
					card[CurrentDevice].OUT.DIGITAL = hod[7];
				}
			}
			strReturn = zprava;
			if (callback) {callback(zprava);}
		}
	})
	return strReturn;
  }
}
